/*===========================================================================*
 * FILE:                        C D E V _ F W . H
 *===========================================================================*
 *
 *   COPYRIGHT 2007 - 2019 ABACO SYSTEMS, INC.  ALL RIGHTS RESERVED.
 *  
 *   THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND COPIED
 *   ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH THE INCLUSION
 *   OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY OTHER COPIES THEREOF
 *   MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY OTHER PERSON.
 *   NO TITLE TO AND OWNERSHIP OF THE SOFTWARE IS HEREBY TRANSFERRED.
 *  
 *   THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT NOTICE
 *   AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS, INC. 
 *
 *===========================================================================*
 *
 * FUNCTION:   Firmware array declaration header file for CEI-x30 products.
 * 
 *---------------------------------------------------------------------------*
 *          
 * HISTORY:   
 *
 *   Date     By   Comments
 * --------  ----  ------------------------------------------------------
 * 12/07/07  BDW   Created.
 * 03/26/08  BDW   Default to uncompressed firmware build, define the
 *                 directive COMPRESSED_FW to use compressed header files.
 * 04/02/09  BDW   Only truly define the CEI-830 firmware load for Integrity
 * 04/07/09  BDW   Remove the use of COMPRESSED_FW and compressed f/w files
 * 09/30/10  BDW   Moved the CEI-530 f/w array delcaration so it is included 
 *                 when compiled for the LabVIEW Real-Time target
 * 09/16/11  BDW   Added CEI-430A firmware array declaration fpga_430A[] by
 *                 including the header file fpga430A.h.
 * 07/10/12  BDW   Added Flashed Firmware Build only support (library size reduction)
 * 10/09/13  BDW   Added support for the RCEI-830A firmware fpga_830a[] by
 *                 including the header file fpga830a.h.
 * 02/07/14  BDW   Added LabVIEW Real-Time support for the RCEI-830A and RCEI-830X820
 * 10/26/16  BDW   Expand INTEGRITY O/S support to include R830RX, RAR-CPCI, CEI-430 & RCEI-530
 *
 *===========================================================================*/

#ifndef CDEV_FW_H
#define CDEV_FW_H

#include "cdev_api_cfg.h"

/*===========================================================================* 
 *
 *  NOTE:  To reduce the overall size of the API, you may eliminate firmware
 *         support for individual unused boards by replacing the standard 
 *
 *            #include "fpga***.h"
 *
 *         with 
 *
 *            #include "fpgax30n.h"
 *
 *         As demonstrated when building for the PXI and LabVIEW Real-Time  
 *         operating systems  (when the directive "NIVISA_PXI" is defined).
 *
 *         * For Integrity, define the directives INTEGRITY_PCI_PPC or 
 *           INTEGRITY_PCI_X86 to eliminate all but the CEI-830, RCEI-830A,
 *           R830RX, RAR-CPCI, CEI-430, and RCEI-530 firmware load content.
 *         * For custom API builds specifically limited to the RAR-PCIE,
 *           RAR-MPCIE & RAR15-XMC-XT, define the directive INCLUDE_PLX_SUPPORT
 *           to selectively eliminate all firmware load data content.
 *
 *===========================================================================*/
#ifndef INCLUDE_PLX_SUPPORT
   /* all firmware loads are defined as 1 element arrays */ 
   static CEI_UCHAR  const fpga_830[] = {
      #include "fpgax30n.h" 
   };
   static CEI_UCHAR  const fpga_830a[] = {
      #include "fpgax30n.h" 
   };
   static CEI_UCHAR  const fpga_830_rx[] = {
      #include "fpgax30n.h" 
   };
   static CEI_UCHAR  const fpga830x820[] = {
      #include "fpgax30n.h" 
   };
   static CEI_UINT32 const fpga_630[] = {
      #include "fpgax30n.h" 
   };
   static CEI_UCHAR  const fpga_430[] = {
      #include "fpgax30n.h" 
   };
   static CEI_UCHAR  const fpga_430A[] = {
      #include "fpgax30n.h" 
   };
   static CEI_UINT32 const fpga_530[] = {
      #include "fpgax30n.h" 
   };
   static CEI_UINT32 const fpga_A30[] = {
      #include "fpgax30n.h" 
   };
   static CEI_UINT32 const fpga_ec[] = {
      #include "fpgax30n.h" 
   };
#else
  #if defined (INTEGRITY_PCI_PPC) || defined (INTEGRITY_PCI_X86)
      static CEI_UCHAR  const fpga_830[] = {
         #include "fpga830.h" 
      };
      static CEI_UCHAR  const fpga_830a[] = {
         #include "fpga830a.h" 
      };
      static CEI_UCHAR  const fpga_830_rx[] = {
         #include "fpga830rx.h" 
      };
      static CEI_UINT32 const fpga_630[] = {
         #include "fpga630.h" 
      };
      static CEI_UCHAR  const fpga_430[] = {
         #include "fpga430.h" 
      };
      static CEI_UINT32 const fpga_530[] = {
         #include "fpga530.h" 
      };
   
      /* For Integrity, all other board firmware is defined as 1 element arrays */ 
      static CEI_UCHAR  const fpga830x820[] = {
         #include "fpgax30n.h" 
      };
      static CEI_UCHAR  const fpga_430A[] = {
         #include "fpgax30n.h" 
      };
      static CEI_UINT32 const fpga_A30[] = {
         #include "fpgax30n.h" 
      };
      static CEI_UINT32 const fpga_ec[] = {
         #include "fpgax30n.h" 
      };
  #else
      
      static CEI_UCHAR  const fpga_830[] = { 
         #include "fpga830.h" 
      };
      static CEI_UCHAR  const fpga_830a[] = {
         #include "fpga830a.h" 
      };
      static CEI_UCHAR  const fpga830x820[] = {
         #include "fpga830x820.h" 
      };
      static CEI_UCHAR  const fpga_830_rx[] = {
         #include "fpga830rx.h" 
      };
      static CEI_UINT32 const fpga_630[] = {
         #include "fpga630.h" 
      };
      static CEI_UINT32 const fpga_530[] = {
         #include "fpga530.h" 
      };
        
      /* For LabVIEW Real-Time, include firmware support for:
       * RCEI-530, CEI-830, RCEI-830A, RCEI-830X820, R830RX and RAR-CPCI firmware above.
       * For the 430, A30 and RAR-EC include a 1 element array instead.
       */
      #if defined (NIVISA_PXI)    
         static CEI_UINT32 const fpga_ec[] = {
            #include "fpgax30n.h" 
         };
         static CEI_UCHAR  const fpga_430[] = {
            #include "fpgax30n.h" 
         };
         static CEI_UCHAR  const fpga_430A[] = {
            #include "fpgax30n.h" 
         };
         static CEI_UINT32 const fpga_A30[] = {
            #include "fpgax30n.h" 
         };
      #else
         /* The real stuff */
         static CEI_UINT32 const fpga_ec[] = {
            #include "fpga_ec.h" 
         };
         static CEI_UCHAR  const fpga_430[] = {
            #include "fpga430.h" 
         };
         static CEI_UCHAR  const fpga_430A[] = {
            #include "fpga430A.h" 
         };
         static CEI_UINT32 const fpga_A30[] = {
            #include "fpgaA30.h" 
         };
      #endif
   #endif
#endif

#endif /* ifndef CDEV_FW_H */

