/*===================================================================================================================*
 * FILE:                              C o n d o r V X W R T P D r v . h
 *===================================================================================================================*
 *
 *   COPYRIGHT (C) 1998-2021 BY ABACO SYSTEMS, INC.  ALL RIGHTS RESERVED.
 *
 *   THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *   COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *   THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *   OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *   AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *   SOFTWARE IS HEREBY TRANSFERRED.
 *
 *   THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *   NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *===================================================================================================================*
 *
 * FUNCTION:   Include file for CondorVXWRTPDrv.c
 *
 *===================================================================================================================*/

/* Revision:   1.22 Release
   Date        Modification Notes
  ----------   ---------------------------------------------------------------
  02/29/2008   Unified PowerPC and x86 installation for Kernel Module and RTP
  08/13/2009   Merged Commerical and 1553 product VxWorks driver - lets be common
  07/19/2010   Added RAR-PCIE and RPCIE-1553 to board_to_map[] and base_region[]
  03/07/2013   Add commercial compatibility
  11/13/2013   Add support for new boards
  12/13/2016   Added support for the Mini-PCIe boards CONDOR_R15_MPCIE & CONDOR_RAR_MPCIE
  10/05/2021   Removing any instance of the character sequences "geip", "gefes", "gef" or related,
               replaced with "avio" unless otherwise noted; included consistency modifications for hump-style
               function names and underscore-delimiter variable names.
*/

#ifndef CONDOR_VXW_RTP_DRV_H
#define CONDOR_VXW_RTP_DRV_H

float CONDOR_VXW_DRV_VER   = 1.22;
char CONDOR_VXW_DRV_DATE[] = "12/13/2016";

#include <errno.h>
#include <semLib.h>
#include <stdio.h>
#include <drv/pci/pciConfigLib.h>
#include <drv/pci/pciIntLib.h>
#include <pci.h>
#include <sysLib.h>
#include <msgQLib.h>
#ifndef VXW_PCI_X86
   #include <vmLib.h>
   #include <private/vmLibP.h>
#endif /* VXW_PCI_X86 */   
#include <vxWorks.h>  
#include <version.h>
#include <stdlib.h>
#include <iosLib.h>  
#include <selectLib.h>
#include "lowlevel.h"
#include "avio_ioctl.h"
#include "cei_types.h"
#ifdef SPIN_LOCK_PROTECT
   #if _WRS_VXWORKS_MAJOR == 6 && _WRS_VXWORKS_MINOR == 6
      #include <spinLockLib.h> 
      spinlockIsr_t spinlock[MAX_BTA];
   #else
      #error Spinlocks not supported by this version of VxWorks
   #endif /* _WRS_VXWORKS_MAJOR == 6 && _WRS_VXWORKS_MINOR == 6 */
#endif /* SPIN_LOCK_PROTECT */

#define CONDOR_ENGINEERING  0x13c6
#define CONDOR_PCI_1553     0x1553
#define CONDOR_ISA_DEV      0x1111
#define CONDOR_AR15_VPX     0x1529
#define CONDOR_QPCI_1553    0x1554
#define CONDOR_QCP_1553     0x1555
#define CONDOR_QPCX_1553    0x1556
#define CONDOR_RXMC_1553    0x1559
#define CONDOR_RPCIE_1553   0x155A
#define CONDOR_LPCIE_1553   0x155B
#define CONDOR_RXMC2_1553   0x155C
#define CONDOR_RAR15_XMC_XT 0x1542
#define CONDOR_R15_PMC      0x1544
#define CONDOR_PMC_D1       0x1001
#define CONDOR_AFDX         0xAFD0
#define CONDOR_CEI_520      0x0520
#define CONDOR_CEI_620      0x0620
#define CONDOR_CEI_820      0x0820
#define CONDOR_CEI_820TX    0x0821
#define CONDOR_CEI_830      0x0830
#define CONDOR_R830RX       0x0831
#define CONDOR_R830X820     0x0832
#define CONDOR_RCEI830A     0x830A
#define CONDOR_CEI_430      0x0430
#define CONDOR_CEI_430A     0x430A
#define CONDOR_CEI_530      0x0530
#define CONDOR_RAR_CPCI     0x0630
#define CONDOR_P708         0x0708
#define CONDOR_RP708        0x708A
#define CONDOR_PSER         0x1004
#define CONDOR_PDIS         0x1006
#define CONDOR_CEI_A30      0x1009
#define CONDOR_RAR_PCIE     0x100B
#define CONDOR_RAR_XMC      0x100C
#define CONDOR_AFDX         0xAFD0
#define CONDOR_RAR_MPCIE    0x100D
#define CONDOR_R15_MPCIE    0x1545
                           
#define UCA32 0x4000

#if defined NON_INTEL_WORD_ORDER  || defined VXW_PCI_PPC
   #define flipw(word) *(word)= (short)(((*(word) >> 8) & 0x00ff) | ((*(word) << 8) & 0xff00))
   #define flip(longword) *(longword) = ((*(longword) >> 16) & 0x0000ffff) | ((*(longword) << 16) & 0xffff0000)
   #define fliplong(longword) *(longword) = (((*longword & 0x000000ff) << 24) | \
                                             ((*longword & 0x0000ff00) << 8)  | \
                                             ((*longword & 0xff0000)   >> 8)  | \
                                             ((*longword & 0xff000000) >> 24))
#else
   #define flipw(word)
   #define flip(longword)
   #define fliplong(longword)
#endif

#if  vxwdebug == 1
   #define dprint(a,b) printf(a,b)
#else
   #define dprint(a,b)
#endif

#define AVIO_DEV_IO_NAME "avioDev"

unsigned board_to_map[] = {CONDOR_CEI_830,      /* CEI-830      */                           
                           CONDOR_RCEI830A,     /* RCEI-830A    */                           
                           CONDOR_R830X820,     /* RCEI-830X820 */                           
                           CONDOR_R830RX,       /* R830RX       */                           
                           CONDOR_CEI_430,      /* CEI-430      */                           
                           CONDOR_CEI_430A,     /* CEI-430A     */                           
                           CONDOR_CEI_530,      /* CEI-530      */                           
                           CONDOR_RAR_CPCI,     /* RAR-CPCI     */                           
                           CONDOR_RAR_PCIE,     /* RAR-PCIE     */                           
                           CONDOR_RAR_XMC,      /* RAR-XMC      */                           
                           CONDOR_P708,         /* P-708        */                           
                           CONDOR_RP708,        /* RP-708       */                           
                           CONDOR_PSER,         /* P-SER        */                           
                           CONDOR_PDIS,         /* P-DIS        */                           
                           CONDOR_PCI_1553,     /* PCI-1553 QPM(C)-1553 AMC-1553 Q104-1553 */
                           CONDOR_QPCI_1553,    /* QPCI-1553    */                           
                           CONDOR_QCP_1553,     /* QCP-1553     */                           
                           CONDOR_QPCX_1553,    /* QPCX-1553    */                           
                           CONDOR_RXMC_1553,    /* RXMC-1553    */                           
                           CONDOR_RPCIE_1553,   /* RPCIE-1553   */                           
                           CONDOR_LPCIE_1553,   /* LPCIE-1553   */                           
                           CONDOR_RXMC2_1553,   /* RXMC2-1553   */                           
                           CONDOR_RAR15_XMC_XT, /* RAR15-XMC-XT */
                           CONDOR_AR15_VPX,     /* AR15-VPX     */                           
                           CONDOR_PMC_D1,       /* PMC-D1       */                           
                           CONDOR_AFDX,         /* AFDX         */                           
                           CONDOR_CEI_520,      /* CEI-520      */                           
                           CONDOR_CEI_620,      /* CEI-620      */                           
                           CONDOR_CEI_820,      /* CEI-820      */                           
                           CONDOR_CEI_820TX,    /* CEI-820TX    */                           
                           CONDOR_R15_MPCIE,    /* R15_MPCIE    */                           
                           CONDOR_RAR_MPCIE     /* RAR_MPCIE    */                           
};

int base_region[] = {2, /* CEI-830      */
                     2, /* RCEI-830A    */
                     2, /* RCEI-830X820 */
                     2, /* R830RX       */
                     2, /* CEI-430      */
                     2, /* CEI-430A     */
                     2, /* CEI-530      */
                     2, /* RAR-CPCI     */
                     0, /* RAR-PCIE     */
                     0, /* RAR-XMC      */
                     2, /* P-708        */
                     2, /* RP-708       */
                     2, /* P-SER        */
                     2, /* P-DIS        */
                     0, /* PCI-1553 QPM(C)-1553 AMC-1553 Q104-1553 */
                     2, /* QPCI-1553    */
                     2, /* QCP-1553     */
                     2, /* QPCX-1553    */
                     0, /* RXMC-1553    */
                     0, /* RPCIE-1553   */
                     0, /* LPCIE-1553   */
                     0, /* RXMC2-1553   */
                     0, /* RAR15-XMC-XT */
                     0, /* AR15-VPX     */
                     2, /* PMC-D1       */
                     2, /* AFDX         */
                     2, /* CEI-520      */
                     2, /* CEI-620      */
                     2, /* CEI-820      */
                     2, /* CEI-820TX    */
                     0, /* R15_MPCIE    */                           
                     0  /* RAR_MPCIE    */                           
};

int num_boards = sizeof(board_to_map)/4;

#define PCI_CFG_COMMAND          0x04
#define PCI_CMD_IO_ENABLE        0x0001   /* IO access enable */
#define PCI_CMD_MEM_ENABLE       0x0002   /* memory access enable */
#define PCI_CMD_MASTER_ENABLE    0x0004   /* bus master enable */
#define PCI_CMD_MON_ENABLE       0x0008   /* monitor special cycles enable */
#define PCI_CMD_WI_ENABLE        0x0010   /* write and invalidate enable */
#define PCI_CMD_SNOOP_ENABLE     0x0020   /* palette snoop enable */
#define PCI_CMD_PERR_ENABLE      0x0040   /* parity error enable */
#define PCI_CMD_WC_ENABLE        0x0080   /* wait cycle enable */
#define PCI_CMD_SERR_ENABLE      0x0100   /* system error enable */
#define PCI_CMD_FBTB_ENABLE      0x0200   /* fast back to back enable */
#define PCI_MEMBASE_MASK         ~0xf     /* mask for memory base address */
#define PCI_BASE_ADDRESS_SPACE   0x01     /* 0 = memory, 1 = I/O */

#define PCI_CFG_VENDOR_ID        0x00
#define PCI_CFG_DEVICE_ID        0x02
#define PCI_CFG_COMMAND          0x04
#define PCI_CFG_STATUS           0x06
#define PCI_CFG_REVISION         0x08
#define PCI_CFG_PROGRAMMING_IF   0x09
#define PCI_CFG_SUBCLASS         0x0a
#define PCI_CFG_CLASS            0x0b
#define PCI_CFG_CACHE_LINE_SIZE  0x0c
#define PCI_CFG_LATENCY_TIMER    0x0d
#define PCI_CFG_HEADER_TYPE      0x0e
#define PCI_CFG_BIST             0x0f
#define PCI_CFG_BASE_ADDRESS_0   0x10
#define PCI_CFG_BASE_ADDRESS_1   0x14
#define PCI_CFG_BASE_ADDRESS_2   0x18
#define PCI_CFG_BASE_ADDRESS_3   0x1c
#define PCI_CFG_BASE_ADDRESS_4   0x20
#define PCI_CFG_BASE_ADDRESS_5   0x24
#define PCI_CFG_CIS              0x28
#define PCI_CFG_SUB_VENDER_ID    0x2c
#define PCI_CFG_SUB_SYSTEM_ID    0x2e
#define PCI_CFG_EXPANSION_ROM    0x30
#define PCI_CFG_CAP_PTR          0x34
#define PCI_CFG_RESERVED_0       0x35
#define PCI_CFG_RESERVED_1       0x38
#define PCI_CFG_DEV_INT_LINE     0x3c
#define PCI_CFG_DEV_INT_PIN      0x3d
#define PCI_CFG_MIN_GRANT        0x3e
#define PCI_CFG_MAX_LATENCY      0x3f
#define PCI_CFG_SPECIAL_USE      0x41
#define PCI_CFG_MODE             0x43

#define SYSTEM_ERROR         10
#define SYSTEM_OK             0

#define PCI_BASE_ADDRESS_0   0x10
#define PCI_BASE_ADDRESS_1   0x14
#define PCI_BASE_ADDRESS_2   0x18
#define PCI_BASE_ADDRESS_3   0x1c
#define PCI_BASE_ADDRESS_4   0x20
#define PCI_BASE_ADDRESS_5   0x24

#define MAX_MEMORY_REGIONS   6
#define PAGE_SIZE            0x1000

#define CARRIER_MAP_A32      0x10   /* Native VME - A32 Addressing        */
#define CARRIER_MAP_A24      0x11   /* Native VME - A24 Addressing        */
#define CARRIER_MAP_A16      0x12   /* Native VME - A16 Addressing        */
#define CARRIER_MAP_A24S     0x13   /* Native VME - A24 VxWorks SUP DATA Addressing */
#define CARRIER_MAP_A32S     0x14   /* Native VME - A32 VxWorks SUP DATA Addressing */
#define CARRIER_MAP_A24SP    0x15   /* Native VME - A24 VxWorks SUP PGM Addressing */
#define CARRIER_MAP_A32SP    0x16   /* Native VME - A32 VxWorks SUP PGM Addressing */
#define CARRIER_MAP_A32UP    0x17   /* Native VME - A32 VxWorks USR PGM Addressing */ 
#define CARRIER_MAP_A24UP    0x18   /* Native VME - A24 VxWorks USR PGM Addressing */ 

#define DRV_INSTALL 1
#define DRV_UNINSTALL 0

#define MMU_RW_FOR_ALL \
  (MMU_ATTR_PROT_SUP_READ | MMU_ATTR_PROT_SUP_WRITE | \
  MMU_ATTR_PROT_USR_READ | MMU_ATTR_PROT_USR_WRITE  | \
  MMU_ATTR_VALID ) /* 0x5b */

#define MMU_VALID_PROT_MASK \
  (MMU_ATTR_PROT_MSK | MMU_ATTR_VALID_MSK) /* 0x7f */


static UINT32 addresses[] = {
   PCI_BASE_ADDRESS_0,
   PCI_BASE_ADDRESS_1,
   PCI_BASE_ADDRESS_2,
   PCI_BASE_ADDRESS_3,
   PCI_BASE_ADDRESS_4,
   PCI_BASE_ADDRESS_5
};

#define PHYSICAL_MAP 0
#define VIRTUAL_MAP 1

unsigned int_level[MAX_BTA];
static unsigned irq_value[MAX_BTA];

#ifdef VXW_X86_MAP_ADD
STATUS sysMmuMapAdd  (void * address, UINT len, UINT initialStateMask, UINT initialState);
#endif /* VXW_X86_MAP_ADD */

int Qboard[MAX_BTA];
static unsigned mq_data[] = {0,1,2,3,4,5,6,7,8,9};

#ifdef _RTP_SHARED_MEM_MAP_
static SEM_ID isrSemaphore[MAX_BTA];
int isrTask_id[MAX_BTA];
void isrProcess(unsigned);
static void intProcess(int idata,int a,int b,int c,int d,int e,int f,int g,int h,int j);
void junk(int idata,int a,int b,int c,int d,int e,int f,int g,int h,int j);
#else /* _RTP_SHARED_MEM_MAP_ */
static void intProcess(unsigned);
#endif /* _RTP_SHARED_MEM_MAP_ */

static int bd_mapped = 0;
static int avio_pci_drv_num=0;  /* every driver gets a driver number when installed */
static int number_of_avio_devices=0;

/* forward declarations - local static functions called though function pointers*/

static STATUS avioPciDrvOpen();
static STATUS avioPciDrvClose();
static STATUS avioPciDrvRead();
static STATUS avioPciDrvWrite();
static STATUS avioPciDrvIoctl();
static STATUS avioPciDrvDelete();
static int    avioPciDrvCreate();

/* Parameter Selection Error checking */
#if defined (_RTP_PHYS_MEM_MAP_) && defined (_RTP_SHARED_MEM_MAP_)
   #error _RTP_SHARED_MEM_MAP_ and _RTP_PHYS_MEM_MAP_ are mutally exclusive parameters
#endif

#if defined (VXW_SYSPCI_MAP) && defined(VXW_SYS_BUS_MAP)
   #error VXW_SYS_PCI_MAP and VXW_SYS_BUS_MAP are mutually exclusive paramters
#endif

#if defined _RTP_SHARED_MEM_MAP_ && (defined (VXW_SYS_PCI_MAP) || defined (VXW_SYS_BUS_MAP) || defined(VXW_X86_MAP_ADD))
   #error Share Memory cannot be used with other mapping
#endif 


typedef struct pciregion_state 
{
   int busNumber;                        /* From Find Device */
   int deviceNumber;                     /* From Find Device */
   int functionNumber;                   /* From Find Device */
   unsigned char irq;                    /* Interrupt Request Level */
   unsigned long size[MAX_MEMORY_REGIONS];
   unsigned long membase[MAX_MEMORY_REGIONS];
   UINT initialStateMask;                /* mask parameter to vmStateSet */
   UINT initialState;                    /* state parameter to vmStateSet */
   int bustype;                          /* ISA = 1 PCI = 2 */
   unsigned short CondorDeviceID;        /* PCI Device ID */
   unsigned short manufacturerID;        /* PCI Manufacturer ID        added by BDW */
   UINT dev_id;                          /* Condor Board ID i.e 0x110 for QPMC-1553 */
   UINT dev_chan;
   unsigned short hif;
   UINT use_count;
   UINT region;
   UINT region_count;
   UINT mem_base_reg;
   UINT num_regions;
   UINT mapping;
   UINT32 VirtAddr[6];
   #ifdef _RTP_SHARED_MEM_MAP_
   SD_ID sd_id[6];
   #endif                                /* _RTP_SHARED_MEM_MAP_ */
   MSG_Q_ID msg_q_id;
} pciregion_state;

static pciregion_state pci_state[MAX_BTA];

#ifndef INT_VEC_IRQ0
   #define INT_VEC_IRQ0    0x00  /* vector for IRQ0 */
#endif

/* the device structure */
typedef struct avio_pci_dev 
{
   DEV_HDR devHdr;                      
   pciregion_state *pci_state;
   UINT read_target;
   UINT use_count;
   UINT thisDevice;                
} AVIO_PCI_DEV;

AVIO_PCI_DEV *p_avio_pci_device[MAX_BTA];

static UINT cardNumberIndex[] = 
  {15, 15, 15, 15,    /* this is the 1553 card number table    */
   15, 15, 15, 15,    /* Its indexed to return the device      */ 
   15, 15, 15, 15,    /* number by card number                 */
   15, 15, 15, 15};

static UINT dev_chan[] = 
  {0, 0, 0, 0,   /* this is the 1553 channel number table      */
   0, 0, 0, 0,   /* it list the number of channels per device  */
   0, 0, 0, 0,   /* its indexed by device number               */ 
   0, 0, 0, 0};

static UINT dev_id[]= 
  {15, 15, 15, 15,     /* This is the device table. it list the    */
   15, 15, 15, 15,     /* device installed.  It indexed by device  */
   15, 15, 15, 15,     /* number.                                  */
   15, 15, 15, 15}; 

static UINT QdevID[] = {0x110,0x120,0x160,0x170,0x180,0x0,0x210,0,0,0,0x260,0x250,0,0x300,0x320};
static UINT  devID[] = {0x40,0x80,0xa0,0x100};

int avioPciDriverControl(int,int);
int avioCfg1553Devices(int);
int mapCondorPCIAddress(void);

#endif /*   CONDOR_VXW_RTP_DRV_H */
