/*====================================================================================================================*
 * FILE:                                         A R _ E R R O R . H
 *====================================================================================================================*
 *
 *   COPYRIGHT 2002 - 2019 ABACO SYSTEMS, INC.  ALL RIGHTS RESERVED.
 *  
 *   THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND COPIED
 *   ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH THE INCLUSION
 *   OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY OTHER COPIES THEREOF
 *   MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY OTHER PERSON.
 *   NO TITLE TO AND OWNERSHIP OF THE SOFTWARE IS HEREBY TRANSFERRED.
 *  
 *   THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT NOTICE
 *   AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS, INC. 
 *
 *====================================================================================================================*
 *
 * FUNCTION:    Error strings for CEI-x30 utilities.
 *
 * HISTORY:
 *
 *   Date    By   Ver. Comments                                                                                        
 * -------- ----- ---- -----------------------------------------------------------------------------------------------*
 * 02/02/09  BDW  2.00 Modified the global error array names.
 * 02/09/12  BDW  4.01 Inserted commas after ar_x30_errors_1[4] and arw_x30_errors[24]
 *                     to correct missing error description string termination.
 * 08/19/12  BDW  4.01 Added description for ARS_FW_NOT_SUPPORTED to ar_x30_errors_2
 * 03/27/15  BDW  4.70 Added descriptions for receiver BIT failures ar_x30_bit_errors
 *                     Renamed all of the constants and arrays from *_830_* to *_X30_*
 *                     Added description for the API return status constant value ARS_RX_BUFFER_OVERRUN
 *
 *====================================================================================================================*/ 
                   
#ifndef AR_ERROR_H
#define AR_ERROR_H

#define MAX_X30_1ST_ERRORS        7
#define MAX_X30_2ND_ERRORS        53
#define MAX_X30_BIT_ERRORS        33
#define MAX_X30_INTFC_ERRORS      23
#define MAX_X30_WMS_ERRORS        26
#define MAX_X30_ERRORS            MAX_X30_2ND_ERRORS  
                         
/*---------------------------------------------------------------------------*
 *  These are general RCVR status codes.
 *---------------------------------------------------------------------------*/

pCEI_CHAR ar_x30_errors_1[MAX_X30_1ST_ERRORS] = {
   "No data received",                     /*  0 */
   "Normal successful completion",         /*  1 */
   "Unknown Error Code",                   /*  2 */
   "Unknown Error Code",                   /*  3 */
   "No Error, Received data",              /*  4 */
   "Invalid ARINC 429 msg detected",       /*  5 */
   "Receive Buffer Overrun detected"       /*  6 */
};
                                                       
/*---------------------------------------------------------------------------*
 *  The following is the standard ARINC error message group.  The 
 *  corresponding values are offset from 1000
 *---------------------------------------------------------------------------*/

CEI_CHAR *ar_x30_errors_2[MAX_X30_2ND_ERRORS] = {
   "Unknown Error Code",                    /*  0 */
   "Unknown Error Code",                    /*  1 */
   "Invalid configuration command",         /*  2 */
   "Invalid configuration value",           /*  3 */
   "Transmit buffer overflow",              /*  4 */
   "Invalid board argument",                /*  5 */
   "Timeout synchronizing with the ARINC",  /*  6 */
   "Firmware load failure",                 /*  7 */
   "Board has not been loaded",             /*  8 */
   "Error initializing timer",              /*  9 */
   "Host CPU can't keep up with data rate", /* 10 */
   "Can't initialize timer on slave",       /* 11 */
   "Memory address error",                  /* 12 */
   "Memory Test failure",                   /* 13 */
   "Invalid string",                        /* 14 */
   "Invalid equipment ID",                  /* 15 */
   "Checksum error loading slave",          /* 16 */
   "No response on receiver",               /* 17 */
   "Translation not available",             /* 18 */
   "Invalid argument",                      /* 19 */
   "Invalid label",                         /* 20 */
   "Driver load failed",                    /* 21 */
   "Device driver error encountered",       /* 22 */
   "Channel timeout on receive function",   /* 23 */
   "Function not supported by driver",      /* 24 */
   "Improperly specified DAC value",        /* 25 */
   "Firmware failure",                      /* 26 */
   "Memory Test failure",                   /* 27 */
   "Illegal Error Code",                    /* 28 */
   "Not programmed for Enhanced Operation", /* 29 */
   "Supported device not detected",         /* 30 */
   "Wrap Test data mismatch failure",       /* 31 */
   "Wrap Test time-tag/baud rate failure",  /* 32 */
   "Incomplete ARINC 708 data frame    ",   /* 33 */
   "Too much data in ARINC 708 frame   ",   /* 34 */
   "BIT terminated due to external data",   /* 35 */
   "Wrap Test missing data failure",        /* 36 */
   "Failed to install/uninstall ISR",       /* 37 */
   "Failed to acquire/release board lock",  /* 38 */
   "Failed to acqire/rel board cfg lock",   /* 39 */
   "Feature not currently supported",       /* 40 */
   "Feature not supported under this o/s",  /* 41 */
   "Unable to grant exclusive load",        /* 42 */
   "Int not supported by board and/or o/s", /* 43 */
   "Int support has not been enabled",      /* 44 */
   "Other process already installed ISR",   /* 45 */
   "Thread waiting for data on this rx ch", /* 46 */
   "Fail waiting for data on this rx chan", /* 47 */
   "Failed to signal waiting thread",       /* 48 */
   "Requested oper not supported on chan",  /* 49 */
   "Failed to access Install Library",      /* 50 */
   "Failed to allocate shared mem region",  /* 51 */
   "API does not support board f/w version" /* 52 */

};
                                   
pCEI_CHAR ar_x30_bit_errors[MAX_X30_BIT_ERRORS] = {
   "Receiver BIT CMD Readback Failure",     /*  0 */
   "Receiver  1 BIT failure detected",      /*  1 */
   "Receiver  2 BIT failure detected",      /*  2 */
   "Receiver  3 BIT failure detected",      /*  3 */
   "Receiver  4 BIT failure detected",      /*  4 */
   "Receiver  5 BIT failure detected",      /*  5 */
   "Receiver  6 BIT failure detected",      /*  6 */
   "Receiver  7 BIT failure detected",      /*  7 */
   "Receiver  8 BIT failure detected",      /*  8 */
   "Receiver  9 BIT failure detected",      /*  9 */
   "Receiver 10 BIT failure detected",      /* 10 */
   "Receiver 11 BIT failure detected",      /* 11 */
   "Receiver 12 BIT failure detected",      /* 12 */
   "Receiver 13 BIT failure detected",      /* 13 */
   "Receiver 14 BIT failure detected",      /* 14 */
   "Receiver 15 BIT failure detected",      /* 15 */
   "Receiver 16 BIT failure detected",      /* 16 */
   "Receiver 17 BIT failure detected",      /* 17 */
   "Receiver 18 BIT failure detected",      /* 18 */
   "Receiver 19 BIT failure detected",      /* 19 */
   "Receiver 20 BIT failure detected",      /* 20 */
   "Receiver 21 BIT failure detected",      /* 21 */
   "Receiver 22 BIT failure detected",      /* 22 */
   "Receiver 23 BIT failure detected",      /* 23 */
   "Receiver 24 BIT failure detected",      /* 24 */
   "Receiver 25 BIT failure detected",      /* 25 */
   "Receiver 26 BIT failure detected",      /* 26 */
   "Receiver 27 BIT failure detected",      /* 27 */
   "Receiver 28 BIT failure detected",      /* 28 */
   "Receiver 29 BIT failure detected",      /* 29 */
   "Receiver 30 BIT failure detected",      /* 30 */
   "Receiver 31 BIT failure detected",      /* 31 */
   "Receiver 32 BIT failure detected"       /* 32 */
};
                                                       
/*---------------------------------------------------------------------------*
 *  These are general interface library failures
 *---------------------------------------------------------------------------*/

pCEI_CHAR ar_830_intfc_errors[MAX_X30_INTFC_ERRORS] = {
   "Library Does Not Exist"      ,         /*  0 */
   "Function Not Available in Library",    /*  1 */
   "Board Not Installed",                  /*  2 */
   "Installed Board in Not Supported",     /*  3 */
   "Unknown Error Code",                   /*  4 */
   "Unknown Error Code",                   /*  5 */
   "Unknown Error Code",                   /*  6 */
   "Unknown Error Code",                   /*  7 */
   "Unknown Error Code",                   /*  8 */
   "Unknown Error Code",                   /*  9 */
   "Unknown Error Code",                   /* 10 */
   "Unknown Error Code",                   /* 11 */
   "Unknown Error Code",                   /* 12 */
   "Unknown Error Code",                   /* 13 */
   "VISA Error - Invalid Object",          /* 14 */
   "VISA Error - Resource Locked",         /* 15 */
   "VISA Error - Invalid Expression",      /* 16 */
   "VISA Error - Resource Not Found",      /* 17 */
   "VISA Error - Invalid Resource Name",   /* 18 */
   "VISA Error - Invalid Access Mode",     /* 19 */
   "Unknown Error Code",                   /* 20 */
   "VISA Error - Timeout Expired",         /* 21 */
   "VISA Error - Deallocation Problem"     /* 22 */
};
                                                       
/*---------------------------------------------------------------------------* 
 *  The following error messages are specific to the Williamsburg API and can 
 *  be ignored for general usage. 
 *---------------------------------------------------------------------------*/

pCEI_CHAR arw_x30_errors[MAX_X30_WMS_ERRORS] =
{
   "Unknown Error Code",                  /* 2000 */
   "ARS_INVVERS",                         /* 2001 */
   "ARS_TIMEOUT",                         /* 2002 */
   "ARS_INVFILE",                         /* 2003 */
   "ARS_INVCOMMVERS",                     /* 2004 */
   "ARS_FILE_FAIL",                       /* 2005 */
   "ARS_INV_CHANNEL",                     /* 2006 */
   "ARS_NCTS",                            /* 2007 */
   "ARS_BUSY",                            /* 2008 */
   "ARS_FILETOOBIG",                      /* 2009 */
   "ARS_NO_RECVR_RESP",                   /* 2010 */
   "ARS_SEQUENCE_ERROR",                  /* 2011 */
   "ARS_WRITE_BUFF_OVERFL",               /* 2012 */
   "ARS_UNEXPECTED_WORD",                 /* 2013 */
   "ARS_INVALID_CASE",                    /* 2014 */
   "ARS_DUPLICATE_LDU",                   /* 2015 */
   "ARS_GOT_ALO",                         /* 2016 */
   "ARS_GOT_TEST",                        /* 2017 */
   "ARS_PATTERN_FAIL",                    /* 2018 */
   "ARS_MEM_ALLOC_ERR",                   /* 2019 */
   "ARS_NOT_OPENED",                      /* 2020 */
   "ARS_EXCESS_NAK",                      /* 2021 */
   "ARS_EXCESS_SYN",                      /* 2022 */
   "ARS_NAK",                             /* 2023 */
   "ARS_SYN",                             /* 2024 */
   "ARS_EOF"                              /* 2025 */
};

#endif /* AR_ERROR_H */

