/*====================================================================================================================*
 * FILE:                                       C D E V _ A P I _ C F G . H
 *====================================================================================================================*
 *
 *   COPYRIGHT 2016 - 2019 ABACO SYSTEMS, INC.  ALL RIGHTS RESERVED.
 *  
 *   THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND COPIED
 *   ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH THE INCLUSION
 *   OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY OTHER COPIES THEREOF
 *   MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY OTHER PERSON.
 *   NO TITLE TO AND OWNERSHIP OF THE SOFTWARE IS HEREBY TRANSFERRED.
 *  
 *   THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT NOTICE
 *   AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS, INC. 
 *
 *====================================================================================================================*
 *
 * FUNCTION:    Header file for controlling the compiled definitions of CEI-x30 Public and Private API function 
 *              prototype definitions in CDEV_API.H (Public) and CDEV_GLB.H (Private).
 * 
 * 
 * DIRECTIONS FOR USE:
 * 
 *              The CEI-x30 API can be customized to support the minimum scope of source code and features desired
 *              for your target system, eliminating the need for code analysis and validation of unnecessary source.
 *              You may add or subtract API source code for CEI-x30 features by selectively including or excluding
 *              C source files (other than CDEV_API.C) relating to the respective functionality contained therein,
 *              (see the Software User's Manual section called "CEIx-30 API C Source Files").  In conjunction with 
 *              the source files included in your build, selective compiliation of the exported API and local
 *              utility function prototypes are defined in CDEV_API.H and CDEV_GLB.H is provided via defines contained
 *              in this file.  The relationship between the directive definitions in this file, the source files
 *              containing the prototype definitions affected by those definitions, and those respective functions
 *              and features are documented below.
 * 
 *              
 *              Note for Certification API build:  If the compiler directive "CERTIFICATION_BUILD" is defined,
 *              the following directives and applicable source code will be excluded from the build:
 *              
 *              INCLUDE_API_UTILITIES - exclude the functions located in CDEV_API_UTILITY.C
 *              INCLUDE_ARINC_717 - exclude the functions located in CDEV_API_A717.C
 *              INCLUDE_IRIG - exclude select processing in CDEV_API.C and the functions located in CDEV_API_IRIG.C
 *              INCLUDE_LEGACY_API - exclude the functions located in CDEV_API_LEGACY_API.C
 *              INCLUDE_PLX_SUPPORT - select processing in CDEV_API.C and the functions located in CDEV_API_PLX_PGM.C
 *              INCLUDE_MSG_SCHEDULER - select processing in CDEV_API.C and the functions located in CDEV_API_SCHED.C
 *              INCLUDE_CFG_FILE - exclude the functions located in CDEV_API_CFG_FILE.C
 *              INCLUDE_EXPANDED_RX - exclude the functions located in CDEV_API_EXP_RX.C
 *              INCLUDE_EXPANDED_TX - exclude the functions located in CDEV_API_EXP_TX.C
 *              INCLUDE_DISCRETE_IO - exclude discrete and differential I/O processing located in CDEV_API.C
 *              
 *              
 *              FUNCTION PROTOTYPES DECLARED FOR "EXPORT" IN THE COMPILATION OF CDEV_API.H ARE 
 *                 BASED ON THE COMPILER DIRECTIVES HEREIN, WITH THE FUNCTIONS THEMSELVES RESIDING IN THE 
 *                 RESPECTIVE SOURCE FILES LISTED BELOW:
 *              
 *              #define INCLUDE_API_UTILITIES - includes the prototypes for the functions located in CDEV_API_UTILITY.C:
 *                 ar_convert_time_to_string
 *                 ar_convert_1553_time_to_string
 *                 ar_get_base_addr
 *                 ar_get_boardname
 *                 ar_get_boardnameLV
 *                 ar_get_boardtype
 *                 ar_get_channel_index_info
 *                 ar_get_config
 *                 ar_get_storage_mode
 *                 ar_query_device
 *                 ar_version
 *              
 *              #define INCLUDE_ARINC_717 - includes the prototypes for the functions located in CDEV_API_A717.C:
 *                 a717_utl_initialize_global_data
 *                 ar_get_573_config
 *                 ar_get_573_frame
 *                 ar_get_transmitter_mode
 *                 ar_put_573_frame
 *                 ar_set_573_config
 *                 ar_set_transmitter_mode
 *              
 *              #define INCLUDE_INTERRUPTS - includes the prototypes for the functions located in CDEV_AI_INTRPT.C:
 *                 cei_utl_interrupt_queue_read
 *                 cei_utl_interrupt_handler
 *                 a429_utl_initialize_int_queue
 *                 a429_utl_process_hw_int
 *                 ar_hw_interrupt_buffer_read
 *                 ar_int_control
 *                 ar_interrupt_queue_read
 *                 ar_set_isr_function
 *                 
 *              #define INCLUDE_IRIG - includes the prototypes for the functions located in CDEV_API_IRIG.C:
 *                 a429_utl_adjust_irig_threshold
 *                 a429_utl_find_dac_threshold                        * these are all private functions
 *                 a429_utl_is_irig_calibrated
 *                 a429_utl_quick_adjust_irig_threshold
 *              
 *              #define INCLUDE_LEGACY_API - includes the prototypes for the functions located in CDEV_API_LEGACY_API.C:
 *                 ar_get_status
 *                 ar_get_timercntl
 *                 ar_get_timercnt
 *                 ar_get_raw_mode
 *                 ar_init_dual_port
 *                 ar_init_slave
 *                 ar_reset_timercnt
 *                 ar_set_raw_mode
 *                 ar_set_config
 *                 ar_set_timerrate
 *                 ar_wait
 *                 ar_int_control
 *                 ar_recreate_parity
 *                 ar_reset_int
 *                 ar_setinterrupts
 *                 ar_setchparms
 *                 ar_xmit_sync
 *                 ar_dump_dp
 *                 ar_force_version
 *                 ar_formatarinclabel 
 *                 ar_formatbinarylabel
 *                 ar_get_errorLV
 *                 ar_get_harris
 *                 ar_int_set
 *                 ar_int_slave
 *                 ar_loadslv_init_disc
 *                 ar_msg_control
 *                 ar_putword2x16
 *                 ar_reformat
 *                 ar_set_arinc_config 
 *                 ar_set_harris
 *                 ar_timetag_control
 *                 ar_unformat
 *                 ar_flush_receiver   
 *              
 *              #define INCLUDE_PLX_SUPPORT - includes the prototypes for the functions located in CDEV_API_PLX_PGM.C:
 *                 a429_utl_enh                                       
 *                 a429_utl_enh_check                                 * these are all private functions
 *                 a429_utl_program_xilinx_9030
 *                 a429_utl_program_xilinx_9030_32bit
 *                 a429_utl_program_xilinx_9056
 *                 a429_utl_program_xilinx_9056_32bit
 *                 a429_utl_plx_eeprom_read
 *                 a429_utl_plx_eeprom_write_enable
 *                 a429_utl_plx_eeprom_write_disable
 *                 a429_utl_plx_eeprom_write
 *                 
 *              #define INCLUDE_LABEL_FILTER - includes the prototypes for the functions located in CDEV_API_RX_FILTER.C:
 *                 ar_enh_label_filter
 *                 ar_getfilter
 *                 ar_get_label_filter
 *                 ar_label_filter
 *                 ar_putfilter
 *              
 *              #define INCLUDE_MSG_SCHEDULER - includes the prototypes for the functions located in CDEV_API_SCHED.C:
 *                 a429_utl_store_message_scheduler_entry
 *                 a429_utl_validate_scheduled_message_parameters
 *                 ar_assign_scheduler_start_offsets
 *                 ar_define_msg
 *                 ar_define_msg_block
 *                 ar_define_msg_block_lv
 *                 ar_modify_msg
 *                 ar_modify_msg_block
 *                 ar_modify_msg_block_lv
 *                 ar_read_scheduled_msg_block
 *                 ar_read_message_schedule_table_entry
 *                 ar_update_message_block
 *              
 *              #define INCLUDE_CFG_FILE - includes the prototypes for the functions located in CDEV_API_CFG_FILE.C:
 *                 ar_config_channels_from_txt_file
 *                 ar_channel_configuration_from_xml_file
 *                 ar_define_messages_from_txt_file                   * Code therein has a dependency on the 
 *                 ar_define_messages_from_xml_file                   * #define for INCLUDE_MSG_SCHEDULER
 *              
 *              #define INCLUDE_EXPANDED_RX - includes the prototypes for the functions located in CDEV_API_EXP_RX.C:
 *                 a429_utl_get_time_msec
 *                 ar_get_data
 *                 ar_get_data_xt
 *                 ar_get_429_message
 *                 ar_get_latest
 *                 ar_getblock
 *                 ar_getnext
 *                 ar_getnextt
 *                 ar_getwordt
 *                 ar_getnext_xt
 *              
 *              #define INCLUDE_EXPANDED_TX - includes the prototypes for the functions located in CDEV_API_EXP_TX.C:
 *                 ar_putblock_multi_chan
 *                 ar_put_429_message
 * 
 *              #define INCLUDE_DISCRETE_IO - includes functionality supporting Discrete I/O processing in the
 *                                            following routines:
 *                 a429_utl_initialize_device
 *                 ar_get_device_config
 *                 ar_set_device_config
 * 
 *--------------------------------------------------------------------------------------------------------------------*
 *          
 * HISTORY:   
 *
 *   Date    By   Ver. Comments
 * -------- ----- ---- -----------------------------------------------------------------------------------------------
 * 10/18/16  BDW  4.90 Initial version.
 * 06/26/17  BDW  4.93 Added an overall Certification firmware API-compliant directive control over source code build
 *                     via use of the CERTIFICATION_BUILD compiler directive; added INCLUDE_DISCRETE_IO to exclude 
 *                     the respective processing in a429_utl_initialize_device and ar_set/get_device_config
 * 
 *====================================================================================================================*/ 
#ifndef CDEV_API_CFG_H
#define CDEV_API_CFG_H

#ifdef  CERTIFICATION_BUILD

#undef INCLUDE_API_UTILITIES
#undef INCLUDE_ARINC_717
#undef INCLUDE_IRIG
#undef INCLUDE_LEGACY_API
#undef INCLUDE_PLX_SUPPORT
#undef INCLUDE_MSG_SCHEDULER
#undef INCLUDE_CFG_FILE
#undef INCLUDE_EXPANDED_RX
#undef INCLUDE_EXPANDED_TX
#undef INCLUDE_DISCRETE_IO

#define INCLUDE_INTERRUPTS
#define INCLUDE_LABEL_FILTER

#else

#define INCLUDE_API_UTILITIES
#define INCLUDE_ARINC_717
#define INCLUDE_INTERRUPTS
#define INCLUDE_IRIG
#define INCLUDE_LEGACY_API
#define INCLUDE_PLX_SUPPORT
#define INCLUDE_LABEL_FILTER
#define INCLUDE_MSG_SCHEDULER
#define INCLUDE_CFG_FILE
#define INCLUDE_EXPANDED_RX
#define INCLUDE_EXPANDED_TX
#define INCLUDE_DISCRETE_IO

#endif

#endif /* CDEV_API_CFG_H */

